From f3b078f09d9eae40453dd2fc18507a598bdfc8bc Mon Sep 17 00:00:00 2001
From: Peter Kirk <peter.kirk@gmx.de>
Date: Sun, 25 May 2008 20:33:46 +0200
Subject: [PATCH] - preliminary init_from_old for the preprocessor

---
 include/speex/speex_preprocess.h |    2 +
 libspeex/preprocess.c            |   58 ++++++++++++++++++++++++++++++++++++++
 2 files changed, 60 insertions(+), 0 deletions(-)

diff --git a/include/speex/speex_preprocess.h b/include/speex/speex_preprocess.h
index f8eef2c..0eee8a4 100644
--- a/include/speex/speex_preprocess.h
+++ b/include/speex/speex_preprocess.h
@@ -64,6 +64,8 @@ typedef struct SpeexPreprocessState_ SpeexPreprocessState;
 */
 SpeexPreprocessState *speex_preprocess_state_init(int frame_size, int sampling_rate);
 
+SpeexPreprocessState *speex_preprocess_state_init_from_old(int frame_size, int sampling_rate_from_old, SpeexPreprocessState *old_st);
+
 /** Destroys a preprocessor state 
  * @param st Preprocessor state to destroy
 */
diff --git a/libspeex/preprocess.c b/libspeex/preprocess.c
index 354af09..6e37487 100644
--- a/libspeex/preprocess.c
+++ b/libspeex/preprocess.c
@@ -530,6 +530,64 @@ EXPORT SpeexPreprocessState *speex_preprocess_state_init(int frame_size, int sam
    return st;
 }
 
+EXPORT SpeexPreprocessState *speex_preprocess_state_init_from_old(int frame_size, int sampling_rate, SpeexPreprocessState *old_st)
+{
+   int i, j, old_N, min_N, N, M;
+   SpeexPreprocessState *st;
+
+   st = speex_preprocess_state_init(frame_size, sampling_rate);
+
+   M = NB_BANDS;
+   N = st->ps_size;
+   old_N = old_st->ps_size;
+   min_N = ((N < old_N) ? N : old_N);
+
+   //N+M ps, noise, echo_noise, residual_echo, reverb_estimate, old_ps, prior, post, gain, gain2, gain_floor, zeta
+   for(i = 0; i < min_N; ++i) {
+      st->noise[i]           = old_st->noise[i];
+      st->echo_noise[i]      = old_st->echo_noise[i];
+      st->residual_echo[i]   = old_st->residual_echo[i];
+      st->reverb_estimate[i] = old_st->reverb_estimate[i];
+      st->old_ps[i]          = old_st->old_ps[i];
+      st->prior[i]           = old_st->prior[i];
+      st->post[i]            = old_st->post[i];
+      st->gain[i]            = old_st->gain[i];
+      st->gain2[i]           = old_st->gain2[i];
+      st->gain_floor[i]      = old_st->gain_floor[i];
+      st->zeta[i]            = old_st->zeta[i];
+   }
+   for(i = 0; i < M; ++i) {
+      st->noise[N+i]           = old_st->noise[old_N+i];
+      st->echo_noise[N+i]      = old_st->echo_noise[old_N+i];
+      st->residual_echo[N+i]   = old_st->residual_echo[old_N+i];
+      st->reverb_estimate[N+i] = old_st->reverb_estimate[old_N+i];
+      st->old_ps[N+i]          = old_st->old_ps[old_N+i];
+      st->prior[N+i]           = old_st->prior[old_N+i];
+      st->post[N+i]            = old_st->post[old_N+i];
+      st->gain[N+i]            = old_st->gain[old_N+i];
+      st->gain2[N+i]           = old_st->gain2[old_N+i];
+      st->gain_floor[N+i]      = old_st->gain_floor[old_N+i];
+      st->zeta[N+i]            = old_st->zeta[old_N+i];
+   }
+   //size N S, Smin, Stmp, update_prob
+   for(i = 0; i < min_N; ++i) {
+      st->S[i]           = old_st->S[i];
+      st->Smin[i]        = old_st->Smin[i];
+      st->Stmp[i]        = old_st->Stmp[i];
+      st->update_prob[i] = old_st->update_prob[i];
+   }
+
+#ifndef FIXED_POINT
+   st->loudness_accum = old_st->loudness_accum;
+   st->loudness       = old_st->loudness;
+   st->agc_gain       = old_st->agc_gain;
+   st->prev_loudness  = old_st->prev_loudness;
+#endif
+   st->nb_adapt       = old_st->nb_adapt;
+   st->min_count      = old_st->min_count;
+   return st;
+}
+
 EXPORT void speex_preprocess_state_destroy(SpeexPreprocessState *st)
 {
    speex_free(st->frame);
-- 
1.5.5.1

